#!/usr/bin/env python3
import re
from urllib.parse import unquote, urlparse, parse_qs
from html import unescape

from .. import Unit
from ...lib.decorators import unicoded


class urlguards(Unit):
    """
    Restores the original URLs from their 'protected' versions as generated by
    Outlook protection and ProofPoint.
    """

    @unicoded
    def process(self, data: str) -> str:

        def proofpoint_replacer(match):
            self.log_info('proofpoint match:', match.group(1))
            argmatch = re.search(r'u=(.+?)&', match.group(2))
            if not argmatch:
                self.log_warn('not able to translate unexpected proofpoint format:', match)
                return match.group(0)
            encoded = argmatch.group(1)
            if match.group(1) == '2':
                encoded = encoded.translate(str.maketrans('-_', '%/'))
            return unescape(unquote(encoded))

        def outlook_replacer(match):
            result = match.group(0)
            self.log_info('outlook match:', result)
            parsed = urlparse(result)
            params = parse_qs(parsed.query)
            try:
                result = unquote(params['url'][0])
            except Exception:
                pass
            return result

        data = re.sub(
            r'https?://urldefense.proofpoint.com/v([12])/url([/_=?#&.,\w\%\-]+)',
            proofpoint_replacer,
            data
        )
        data = re.sub(
            r'https?://\w+.safelinks.protection.outlook.com/([/_=\?#&.,\w\%\-]+)',
            outlook_replacer,
            data
        )
        return data
